@@ -102,7 +102,7 @@ out_no_root:
 		iput(inode);
 
 out_mount_failed:
-    sb->s_dev = 0;
+	sb->s_dev = 0;
 	return 0;
 }
 
@@ -175,8 +175,10 @@ cifs_alloc_inode(struct super_block *sb)
 	cifs_inode->cifsAttrs = 0x20;	/* default */
 	atomic_set(&cifs_inode->inUse, 0);
 	cifs_inode->time = 0;
-    if(oplockEnabled)
+	if(oplockEnabled) {
+	    cifs_inode->clientCanCacheRead = 1;
 	    cifs_inode->clientCanCacheAll = 1;
+	}
 	INIT_LIST_HEAD(&cifs_inode->openFileList);
 	return &cifs_inode->vfs_inode;
 }
