@@ -1101,7 +1101,7 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 			     ("Count: %d  End: %d ", findParms.SearchCount,
 			      findParms.EndofSearch));
 		} else {
-			cERROR(1,("Search rewinding on %s",full_path));
+			cFYI(1,("Search rewinding on %s",full_path));
 			goto readdir_rewind;
 		}
 
@@ -1113,7 +1113,7 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 					 GFP_KERNEL);
 			else {
 				/* BB close search handle */
-				cERROR(1,("Search rewinding on %s",full_path));
+				cFYI(1,("Search rewinding on %s",full_path));
 			}
 			if (file->private_data) {
 				memset(file->private_data, 0,
