@@ -28,6 +28,26 @@
 #include "cifs_debug.h"
 #include "cifs_fs_sb.h"
 
+struct inode * get_cifs_inode(struct super_block * sb)
+{
+	struct inode * newinode;
+	newinode = new_inode(sb);
+	cFYI(1,("got new inode %p",newinode));
+	if(newinode) {    
+		struct cifsInodeInfo * cifsInfo = CIFS_I(newinode);
+		if(oplockEnabled) {
+			cifsInfo->clientCanCacheRead = 1;
+			cifsInfo->clientCanCacheAll = 1;
+		}
+		INIT_LIST_HEAD(&cifsInfo->openFileList);
+		cifsInfo->cifsAttrs = 0x20;   /* default */
+		atomic_set(&cifsInfo->inUse, 0);
+		cifsInfo->time = 0;
+	}
+	return newinode;
+
+}
+
 int
 cifs_get_inode_info_unix(struct inode **pinode,
 			 const unsigned char *search_path,
@@ -84,12 +104,9 @@ cifs_get_inode_info_unix(struct inode **pinode,
 
 		/* get new inode */
 		if (*pinode == NULL) {
-			*pinode = new_inode(sb);
-			cFYI(1, (" Alloc new inode %p ", *pinode));
+			*pinode = get_cifs_inode(sb);
 		}
 		inode = *pinode;
-/*        new_inode = iget(parent_dir_inode->i_sb, findData.IndexNumber); */
-        /* index number not reliable in response data */
 
 		cifsInfo = CIFS_I(inode);
 
@@ -213,8 +230,7 @@ cifs_get_inode_info(struct inode **pinode,
 
 		/* get new inode */
 		if (*pinode == NULL) {
-			*pinode = new_inode(sb);
-			cFYI(1, (" Alloc new inode %p ", *pinode));
+			*pinode = get_cifs_inode(sb);
 		}
 
 		inode = *pinode;
@@ -665,7 +681,7 @@ cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 
 	if ((cifs_sb->tcon->ses->capabilities & CAP_UNIX)
 	    && (attrs->ia_valid & (ATTR_MODE | ATTR_GID | ATTR_UID)))
-		CIFSSMBUnixSetPerms(xid, pTcon, full_path, mode, uid, gid,
+		rc = CIFSSMBUnixSetPerms(xid, pTcon, full_path, mode, uid, gid,
 				    cifs_sb->local_nls);
 	else {			/* BB to be implemented - via Windows security descriptors */
 		/* CIFSSMBWinSetPerms(xid,pTcon,full_path,mode,uid,gid,cifs_sb->local_nls);*/
@@ -702,7 +718,10 @@ cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 		rc = CIFSSMBSetTimes(xid, pTcon, full_path, &time_buf,
 				     cifs_sb->local_nls);
 	}
-/*	cifsInode->time = 0; */ /* force revalidate to get attributes when needed */
+	
+	/* do not  need local check to inode_check_ok since the server does that */
+	inode_setattr(direntry->d_inode, attrs);
+
 
 	if (full_path)
 		kfree(full_path);
