@@ -38,13 +38,12 @@
 #include "cifs_debug.h"
 #include "nterr.h"
 
-typedef const struct {
+struct smb_to_posix_error {
 	__u16 smb_err;
 	int posix_code;
-} smb_to_posix_error;
+};
 
-
-smb_to_posix_error mapping_table_ERRDOS[] = {
+const struct smb_to_posix_error mapping_table_ERRDOS[] = {
 	{ERRbadfunc, -EINVAL},
 	{ERRbadfile, -ENOENT},
 	{ERRbadpath, -ENOENT},
@@ -77,7 +76,7 @@ smb_to_posix_error mapping_table_ERRDOS[] = {
 	{0, 0}
 };
 
-smb_to_posix_error mapping_table_ERRSRV[] = {
+const struct smb_to_posix_error mapping_table_ERRSRV[] = {
 	{ERRerror, -EIO},
 	{ERRbadpw, -EACCES},
 	{ERRbadtype, -EREMOTE},
@@ -116,18 +115,17 @@ smb_to_posix_error mapping_table_ERRSRV[] = {
 	{0, 0}
 };
 
-smb_to_posix_error mapping_table_ERRHRD[] = {
+const struct smb_to_posix_error mapping_table_ERRHRD[] = {
 	{0, 0}
 };
 
-
-
 /* Convert string containing dotted ip address to binary form */
 /* returns 0 if invalid address */
 
 /* BB add address family, change rc to status flag and return *//* also see inet_pton */
 /* To identify v4 vs. v6 - 1) check for colon (v6 only) 2) then call inet_pton to parse for bad address  */
-int inet_addr(char *cp)
+int
+inet_addr(char *cp)
 {
 	struct in_addr address;
 	int value;
@@ -189,7 +187,7 @@ int inet_addr(char *cp)
 convert a NT status code to a dos class/code
  *****************************************************************************/
 /* NT status -> dos error map */
-const static struct {
+static const struct {
 	__u8 dos_class;
 	__u16 dos_code;
 	__u32 ntstatus;
@@ -254,7 +252,7 @@ const static struct {
 	ERRHRD, ERRgeneral, NT_STATUS_DISK_CORRUPT_ERROR}, {
 	ERRDOS, ERRbadfile, NT_STATUS_OBJECT_NAME_INVALID}, {	/* mapping changed since shell does lookup on * and expects file not found */
 	ERRDOS, ERRbadfile, NT_STATUS_OBJECT_NAME_NOT_FOUND}, {
-	ERRDOS, 183, NT_STATUS_OBJECT_NAME_COLLISION}, {
+	ERRDOS, ERRalreadyexists, NT_STATUS_OBJECT_NAME_COLLISION}, {
 	ERRHRD, ERRgeneral, NT_STATUS_HANDLE_NOT_WAITABLE}, {
 	ERRDOS, ERRbadfid, NT_STATUS_PORT_DISCONNECTED}, {
 	ERRHRD, ERRgeneral, NT_STATUS_DEVICE_ALREADY_ATTACHED}, {
@@ -287,7 +285,7 @@ const static struct {
 	ERRHRD, ERRgeneral, NT_STATUS_EA_CORRUPT_ERROR}, {
 	ERRDOS, ERRlock, NT_STATUS_FILE_LOCK_CONFLICT}, {
 	ERRDOS, ERRlock, NT_STATUS_LOCK_NOT_GRANTED}, {
-	ERRDOS, ERRnoaccess, NT_STATUS_DELETE_PENDING}, {
+	ERRDOS, ERRbadfile, NT_STATUS_DELETE_PENDING}, {
 	ERRDOS, ERRunsup, NT_STATUS_CTL_FILE_NOT_SUPPORTED}, {
 	ERRHRD, ERRgeneral, NT_STATUS_UNKNOWN_REVISION}, {
 	ERRHRD, ERRgeneral, NT_STATUS_REVISION_MISMATCH}, {
@@ -625,10 +623,8 @@ const static struct {
 	ERRHRD, ERRgeneral, NT_STATUS_NETWORK_CREDENTIAL_CONFLICT}, {
 	ERRHRD, ERRgeneral, NT_STATUS_REMOTE_SESSION_LIMIT}, {
 	ERRHRD, ERRgeneral, NT_STATUS_EVENTLOG_FILE_CHANGED}, {
-	ERRDOS, ERRnoaccess, NT_STATUS_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT},
-	{
-	ERRDOS, ERRnoaccess, NT_STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT},
-	{
+	ERRDOS, ERRnoaccess, NT_STATUS_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT}, {
+	ERRDOS, ERRnoaccess, NT_STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT}, {
 	ERRDOS, ERRnoaccess, NT_STATUS_NOLOGON_SERVER_TRUST_ACCOUNT},
 /*	{ This NT error code was 'sqashed'
 	 from NT_STATUS_DOMAIN_TRUST_INCONSISTENT to NT_STATUS_LOGON_FAILURE 
@@ -669,8 +665,7 @@ const static struct {
 	ERRHRD, ERRgeneral, NT_STATUS_MAPPED_ALIGNMENT}, {
 	ERRDOS, 193, NT_STATUS_IMAGE_CHECKSUM_MISMATCH}, {
 	ERRHRD, ERRgeneral, NT_STATUS_LOST_WRITEBEHIND_DATA}, {
-	ERRHRD, ERRgeneral, NT_STATUS_CLIENT_SERVER_PARAMETERS_INVALID},
-	{
+	ERRHRD, ERRgeneral, NT_STATUS_CLIENT_SERVER_PARAMETERS_INVALID}, {
 	ERRSRV, 2242, NT_STATUS_PASSWORD_MUST_CHANGE}, {
 	ERRHRD, ERRgeneral, NT_STATUS_NOT_FOUND}, {
 	ERRHRD, ERRgeneral, NT_STATUS_NOT_TINY_STREAM}, {
@@ -750,7 +745,28 @@ const static struct {
 	ERRDOS, ERRnoaccess, 0xc0000290}, {
 ERRDOS, ERRbadfunc, 0xc000029c},};
 
-void ntstatus_to_dos(__u32 ntstatus, __u8 * eclass, __u16 * ecode)
+/*****************************************************************************
+ Print an error message from the status code
+ *****************************************************************************/
+static void
+cifs_print_status(__u32 status_code)
+{
+	int idx = 0;
+
+	while (nt_errs[idx].nt_errstr != NULL) {
+		if (((nt_errs[idx].nt_errcode) & 0xFFFFFF) ==
+		    (status_code & 0xFFFFFF)) {
+			printk(KERN_NOTICE "Status code returned 0x%08x %s\n",
+				   status_code,nt_errs[idx].nt_errstr);
+		}
+		idx++;
+	}
+	return;
+}
+
+
+static void
+ntstatus_to_dos(__u32 ntstatus, __u8 * eclass, __u16 * ecode)
 {
 	int i;
 	if (ntstatus == 0) {
@@ -769,8 +785,8 @@ void ntstatus_to_dos(__u32 ntstatus, __u8 * eclass, __u16 * ecode)
 	*ecode = ERRgeneral;
 }
 
-
-int map_smb_to_linux_error(struct smb_hdr *smb)
+int
+map_smb_to_linux_error(struct smb_hdr *smb)
 {
 	int i;
 	int rc = -EIO;		/* if transport error smb error may not be set */
@@ -785,10 +801,9 @@ int map_smb_to_linux_error(struct smb_hdr *smb)
 
 	if (smb->Flags2 & SMBFLG2_ERR_STATUS) {
 		/* translate the newer STATUS codes to old style errors and then to POSIX errors */
-		cFYI(1,
-		     (" !!Mapping cifs error code %d ",
-		      smb->Status.CifsError));
 		smb->Status.CifsError = le32_to_cpu(smb->Status.CifsError);
+		if(cifsFYI)
+			cifs_print_status(smb->Status.CifsError);
 		ntstatus_to_dos(smb->Status.CifsError, &smberrclass,
 				&smberrcode);
 	} else {
@@ -799,20 +814,16 @@ int map_smb_to_linux_error(struct smb_hdr *smb)
 	}
 
 	/* old style errors */
-	cFYI(1, (" !!Mapping smb error code %d ", smberrcode));
+
 	/* DOS class smb error codes - map DOS */
-
-	/* BB special case reconnect tid and reconnect uid here? */
-
 	if (smberrclass == ERRDOS) {	/* one byte field no need to byte reverse */
 		for (i = 0;
 		     i <
-		     sizeof(mapping_table_ERRDOS) /
-		     sizeof(smb_to_posix_error); i++) {
+		     sizeof (mapping_table_ERRDOS) /
+		     sizeof (struct smb_to_posix_error); i++) {
 			if (mapping_table_ERRDOS[i].smb_err == 0)
 				break;
-			else if (mapping_table_ERRDOS[i].smb_err ==
-				 smberrcode) {
+			else if (mapping_table_ERRDOS[i].smb_err == smberrcode) {
 				rc = mapping_table_ERRDOS[i].posix_code;
 				break;
 			}
@@ -821,12 +832,11 @@ int map_smb_to_linux_error(struct smb_hdr *smb)
 	} else if (smberrclass == ERRSRV) {	/* server class of error codes */
 		for (i = 0;
 		     i <
-		     sizeof(mapping_table_ERRSRV) /
-		     sizeof(smb_to_posix_error); i++) {
+		     sizeof (mapping_table_ERRSRV) /
+		     sizeof (struct smb_to_posix_error); i++) {
 			if (mapping_table_ERRSRV[i].smb_err == 0)
 				break;
-			else if (mapping_table_ERRSRV[i].smb_err ==
-				 smberrcode) {
+			else if (mapping_table_ERRSRV[i].smb_err == smberrcode) {
 				rc = mapping_table_ERRSRV[i].posix_code;
 				break;
 			}
@@ -835,11 +845,9 @@ int map_smb_to_linux_error(struct smb_hdr *smb)
 	}
 	/* else ERRHRD class errors or junk  - return EIO */
 
-	/* BB get smb->error_class and code lookup in table if ERR_STATUS is not set in this frame 
-	   else translate newer NT Status code - in both cases change to equivalent posix error BB */
-	cFYI(1, (" to POSIX err %d !!", rc));
+	cFYI(1, (" !!Mapping smb error code %d to POSIX err %d !!", smberrcode,rc));
 
-	/* generic corrective action e.g. reconnect SMB session on ERRbaduid - could be added here */
+	/* generic corrective action e.g. reconnect SMB session on ERRbaduid could be added */
 
 	return rc;
 }
@@ -848,14 +856,10 @@ int map_smb_to_linux_error(struct smb_hdr *smb)
  * calculate the size of the SMB message based on the fixed header
  * portion, the number of word parameters and the data portion of the message
  */
-int smbCalcSize(SMB_HDR * ptr)
+int
+smbCalcSize(struct smb_hdr *ptr)
 {
-/*  char *bccPtr;
-  SMB_HDR *ptr2 = (SMB_HDR *) ptr;
-  bccPtr = (char *) &ptr2->WordCount + 1 + (ptr2->WordCount) * 2;
-  return (bccPtr - (char *) ptr + GETU16 (bccPtr) + 2); */
-
-	return (sizeof(struct smb_hdr) + (int) (2 * ptr->WordCount) +
+	return (sizeof (struct smb_hdr) + (int) (2 * ptr->WordCount) +
 		BCC(ptr));
 }
 
@@ -867,7 +871,8 @@ int smbCalcSize(SMB_HDR * ptr)
      * Convert the NT UTC (based 1601-01-01, in hundred nanosecond units)
      * into Unix UTC (based 1970-01-01, in seconds).
      */
-time_t cifs_NTtimeToUnix(u64 ntutc)
+time_t
+cifs_NTtimeToUnix(u64 ntutc)
 {
 	/* BB what about the timezone? BB */
 
@@ -876,12 +881,13 @@ time_t cifs_NTtimeToUnix(u64 ntutc)
 
 	t = ntutc - NTFS_TIME_OFFSET;
 	do_div(t, 10000000);
-	return (time_t) t;
+	return (time_t)t;
 }
 
 /* Convert the Unix UTC into NT UTC. */
-u64 cifs_UnixTimeToNT(time_t t)
+u64
+cifs_UnixTimeToNT(time_t t)
 {
 	/* Convert to 100ns intervals and then add the NTFS time offset. */
-	return (u64) t *10000000 + NTFS_TIME_OFFSET;
+	return (u64) (t * 10000000) + NTFS_TIME_OFFSET;
 }
